/*----------------------------------------------------------------------------------*/
/*==================================================================================*/
/*==   This software is designed to be compiled by the Hi-Tech PIC C compiler     ==*/
/*==================================================================================*/
/*----------------------------------------------------------------------------------*/
/*                                                                                  */
/*----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------*/

/*  History:                                                                        
    --------  
	2008-01-13	Thanks to Jussi... He has encountered a PCD8544 compatible display that uses a chip that can address a larger screen. In order to keep compatible
				he suggested a few changes in routines: LCD_clr, Lcd_show_1541_III and Lcd_show_1541_III_DTV
				these routines have an extra set of commands that resets the pointer to the beginnng of the next line instead of
				dumping the image pixel for pixel to the LCD's memory. This makes the software more compatible with PCD8544 LCD controller clones
	2008-01-12	made 2 different types of LCD_bar functions ProgressBar and SelectionBar
	2007-10-27	by making the data-line 0 after a bit is send, we don't we'd get contrast problems when we use to much black i.e. a line of inverted characters would blank out the entire display
	2007-10-25	removed the SX splash screen as my 1541-III inside my SX is nothing more then a v1.0 upgraded to a v1.1
	2006-10-02	added a 1541-III DTV splash screen
	2006-08-23	added font setting, now the bold section of the 1541-III font can be used in a simpler way
	2006-08-17	bug found: skipped the first byte of the splash image, although it seemed OK, a problem showed at the last 30% of the picture but in fact the WHOLE	PICTURE was one byte short
	2006-04-20	small improvement in readabillity of inverted char-mode
	2006-04-03	added an independent LCD_clr routine and Lcd_show_image (for splash screen
	2006-03-31	2nd attempt, it works (problem was the hardware (connector problem))
	2006-03-30	1st attempt...
*/

/*  TO DO:                                                                          
    ------      
	- ...
*/

/*----------------------------------------------------------------------------------*/
/*                                                                                  */
/* +-------------------------+                                                      */
/* |     1 2 3 4 5 6 7 8     |                                                      */
/* |     # # # # # # # #     |                                                      */
/* |  ===#=#=#=#=#=#=#=#===  |  1 =  VDD  +V (? mA) Chip power supply               */
/* +--=====================--+  2 =  SCLK serial clock line of LCD                  */
/* |                         |  3 =  SI   serial data input of LCD                  */
/* |     (Nokia 3310 LCD)    |  4 =  D/C  command/data switch                       */
/* |        rear view        |  5 =  /CS  active low chip select                    */
/* |  connector is visible   |  6 =  GND  Ground for printer port and VDD           */
/* |                         |  7 =  Vout of display-internal dc/dc converter       */
/* |         PCD8544         |  8 =  /RES active low reset                          */
/* +-------------------------+                                                      */
/*                                                                                  */
/*----------------------------------------------------------------------------------*/

/*--------------------------------------------------------*/
/*                        includes                        */
/*--------------------------------------------------------*/
#include <pic18.h>
#include <stdio.h>
#include <delay.h>
#include <hardware.h>
#include <Lcd_Nokia_3310.h>

/*--------------------------------------------------------*/
/*                        constants                       */   
/*--------------------------------------------------------*/
#define			FALSE				0			/*FALSE*/
#define			TRUE				1			/*TRUE*/



//*******************************************
// CCS Formatted Character Font
// Vertical Character Format
//*******************************************
const unsigned char CharFont[256][5]=
0x00,0xfe,0x81,0xfe,0x00,         // 0 	[0x0]
0x00,0xfe,0xc1,0xfe,0x00,         // 1 	[0x1]
0x00,0xfe,0xe1,0xfe,0x00,         // 2 	[0x2]
0x00,0xfe,0xf1,0xfe,0x00,         // 3 	[0x3]
0x00,0xfe,0xf9,0xfe,0x00,         // 4 	[0x4]
0x00,0xfe,0xfd,0xfe,0x00,         // 5 	[0x5]
0x00,0xfe,0xff,0xfe,0x00,         // 6 	[0x6]
0x18,0x1f,0xc1,0xf8,0x08,         // 7 	[0x7]
0x18,0x42,0x3c,0x81,0x7e,         // 8 	[0x8]
0x32,0x2a,0xe7,0x2a,0x32,         // 9 	[0x9]
0xe2,0x51,0xe2,0x64,0x92,         // 10 	[0xa]
0xf5,0x91,0x65,0x61,0x95,         // 11 	[0xb]
0x5c,0x62,0x02,0x62,0x5c,         // 12 	[0xc]
0xfa,0x00,0xf0,0x90,0xfe,         // 13 	[0xd]
0xfe,0x92,0x00,0xfe,0x02,         // 14 	[0xe]
0x7c,0x7c,0x6c,0x7c,0x7c,         // 15 	[0xf]
0x00,0xfe,0x7c,0x38,0x10,         // 16 	[0x10]
0x10,0x38,0x7c,0xfe,0x00,         // 17 	[0x11]
0x40,0x70,0x7c,0x70,0x40,         // 18 	[0x12]
0x04,0x1c,0x7c,0x1c,0x04,         // 19 	[0x13]
0x88,0xcc,0xee,0xcc,0x88,         // 20 	[0x14]
0x22,0x66,0xee,0x66,0x22,         // 21 	[0x15]
0x38,0x7c,0x7c,0x7c,0x38,         // 22 	[0x16]
0x3e,0x3e,0x3e,0x3e,0x3e,         // 23 	[0x17]
0x20,0x70,0xa8,0x20,0x3e,         // 24 	[0x18]
0x08,0x04,0xfe,0x04,0x08,         // 25 	[0x19]
0x20,0x40,0xfe,0x40,0x20,         // 26 	[0x1a]
0x10,0x10,0x54,0x38,0x10,         // 27 	[0x1b]
0x10,0x38,0x54,0x10,0x10,         // 28 	[0x1c]
0x80,0x88,0x94,0xa2,0x80,         // 29 	[0x1d]
0x80,0xa2,0x94,0x88,0x80,         // 30 	[0x1e]
0xfe,0x82,0x82,0x84,0xf8,         // 31 	[0x1f]
0x00,0x00,0x00,0x00,0x00,         // 32 	[0x20]
0x00,0x00,0xbe,0x00,0x00,         // 33 	[0x21]
0x00,0x0e,0x00,0x0e,0x00,         // 34 	[0x22]
0x28,0xfe,0x28,0xfe,0x28,         // 35 	[0x23]
0x48,0x54,0xfe,0x54,0x24,         // 36 	[0x24]
0x46,0x26,0x10,0xc8,0xc4,         // 37 	[0x25]
0x6c,0x92,0xaa,0x44,0xa0,         // 38 	[0x26]
0x00,0x00,0x0a,0x06,0x00,         // 39 	[0x27]
0x00,0x38,0x44,0x82,0x00,         // 40 	[0x28]
0x00,0x82,0x44,0x38,0x00,         // 41 	[0x29]
0x28,0x10,0x7c,0x10,0x28,         // 42 	[0x2a]
0x10,0x10,0x7c,0x10,0x10,         // 43 	[0x2b]
0x00,0xa0,0x60,0x00,0x00,         // 44 	[0x2c]
0x10,0x10,0x10,0x10,0x10,         // 45 	[0x2d]
0x00,0xc0,0xc0,0x00,0x00,         // 46 	[0x2e]
0x40,0x20,0x10,0x08,0x04,         // 47 	[0x2f]
0x7c,0xa2,0x92,0x8a,0x7c,         // 48 	[0x30]
0x00,0x84,0xfe,0x80,0x00,         // 49 	[0x31]
0x84,0xc2,0xa2,0x92,0x8c,         // 50 	[0x32]
0x42,0x82,0x8a,0x96,0x62,         // 51 	[0x33]
0x30,0x28,0x24,0xfe,0x20,         // 52 	[0x34]
0x4e,0x8a,0x8a,0x8a,0x72,         // 53 	[0x35]
0x78,0x94,0x92,0x92,0x60,         // 54 	[0x36]
0x06,0x02,0xe2,0x12,0x0e,         // 55 	[0x37]
0x6c,0x92,0x92,0x92,0x6c,         // 56 	[0x38]
0x0c,0x92,0x92,0x52,0x3c,         // 57 	[0x39]
0x00,0xcc,0xcc,0x00,0x00,         // 58 	[0x3a]
0x00,0xac,0x6c,0x00,0x00,         // 59 	[0x3b]
0x10,0x28,0x44,0x82,0x00,         // 60 	[0x3c]
0x28,0x28,0x28,0x28,0x28,         // 61 	[0x3d]
0x00,0x82,0x44,0x28,0x10,         // 62 	[0x3e]
0x04,0x02,0xa2,0x12,0x0c,         // 63 	[0x3f]
0x64,0x92,0xf2,0x82,0x7c,         // 64 	[0x40]
0xf8,0x24,0x22,0x24,0xf8,         // 65 	[0x41]
0xfe,0x92,0x92,0x92,0x6c,         // 66 	[0x42]
0x7c,0x82,0x82,0x82,0x44,         // 67 	[0x43]
0xfe,0x82,0x82,0x44,0x38,         // 68 	[0x44]
0xfe,0x92,0x92,0x92,0x82,         // 69 	[0x45]
0xfe,0x12,0x12,0x12,0x02,         // 70 	[0x46]
0x7c,0x82,0x92,0x92,0xf4,         // 71 	[0x47]
0xfe,0x10,0x10,0x10,0xfe,         // 72 	[0x48]
0x00,0x82,0xfe,0x82,0x00,         // 73 	[0x49]
0x40,0x80,0x82,0x7e,0x02,         // 74 	[0x4a]
0xfe,0x10,0x28,0x44,0x82,         // 75 	[0x4b]
0xfe,0x80,0x80,0x80,0x80,         // 76 	[0x4c]
0xfe,0x04,0x18,0x04,0xfe,         // 77 	[0x4d]
0xfe,0x08,0x10,0x20,0xfe,         // 78 	[0x4e]
0x7c,0x82,0x82,0x82,0x7c,         // 79 	[0x4f]
0xfe,0x12,0x12,0x12,0x0c,         // 80 	[0x50]
0x7c,0x82,0xa2,0x42,0xbc,         // 81 	[0x51]
0xfe,0x12,0x32,0x52,0x8c,         // 82 	[0x52]
0x4c,0x92,0x92,0x92,0x64,         // 83 	[0x53]
0x02,0x02,0xfe,0x02,0x02,         // 84 	[0x54]
0x7e,0x80,0x80,0x80,0x7e,         // 85 	[0x55]
0x3e,0x40,0x80,0x40,0x3e,         // 86 	[0x56]
0xfe,0x40,0x30,0x40,0xfe,         // 87 	[0x57]
0xc6,0x28,0x10,0x28,0xc6,         // 88 	[0x58]
0x0e,0x10,0xe0,0x10,0x0e,         // 89 	[0x59]
0xc2,0xa2,0x92,0x8a,0x86,         // 90 	[0x5a]
0x00,0xfe,0x82,0x82,0x00,         // 91 	[0x5b]
0x04,0x08,0x10,0x20,0x40,         // 92 	[0x5c]
0x00,0x82,0x82,0xfe,0x00,         // 93 	[0x5d]
0x08,0x04,0x02,0x04,0x08,         // 94 	[0x5e]
0x80,0x80,0x80,0x80,0x80,         // 95 	[0x5f]
0x00,0x02,0x04,0x08,0x00,         // 96 	[0x60]
0x40,0xa8,0xa8,0xa8,0xf0,         // 97 	[0x61]
0xfe,0x90,0x88,0x88,0x70,         // 98 	[0x62]
0x70,0x88,0x88,0x88,0x40,         // 99 	[0x63]
0x70,0x88,0x88,0x90,0xfe,         // 100 	[0x64]
0x70,0xa8,0xa8,0xa8,0x30,         // 101 	[0x65]
0x10,0xfc,0x12,0x02,0x04,         // 102 	[0x66]
0x10,0xa8,0xa8,0xa8,0x78,         // 103 	[0x67]
0xfe,0x10,0x08,0x08,0xf0,         // 104 	[0x68]
0x00,0x90,0xfa,0x80,0x00,         // 105 	[0x69]
0x40,0x80,0x88,0x7a,0x00,         // 106 	[0x6a]
0x00,0xfe,0x20,0x50,0x88,         // 107 	[0x6b]
0x00,0x82,0xfe,0x80,0x00,         // 108 	[0x6c]
0xf8,0x08,0xf0,0x08,0xf8,         // 109 	[0x6d]
0xf8,0x10,0x08,0x08,0xf0,         // 110 	[0x6e]
0x70,0x88,0x88,0x88,0x70,         // 111 	[0x6f]
0xf8,0x28,0x28,0x28,0x10,         // 112 	[0x70]
0x08,0x14,0x14,0x18,0xfc,         // 113 	[0x71]
0xf8,0x10,0x08,0x08,0x10,         // 114 	[0x72]
0x90,0xa8,0xa8,0xa8,0x40,         // 115 	[0x73]
0x08,0x7e,0x88,0x80,0x40,         // 116 	[0x74]
0x78,0x80,0x80,0x40,0xf8,         // 117 	[0x75]
0x38,0x40,0x80,0x40,0x38,         // 118 	[0x76]
0x78,0x80,0x60,0x80,0x78,         // 119 	[0x77]
0x88,0x50,0x20,0x50,0x88,         // 120 	[0x78]
0x18,0xa0,0xa0,0xa0,0x78,         // 121 	[0x79]
0x88,0xc8,0xa8,0x98,0x88,         // 122 	[0x7a]
0x00,0x10,0x6c,0x82,0x00,         // 123 	[0x7b]
0x00,0x00,0xfe,0x00,0x00,         // 124 	[0x7c]
0x00,0x82,0x6c,0x10,0x00,         // 125 	[0x7d]
0x04,0x02,0x04,0x08,0x04,         // 126 	[0x7e]
0xf0,0x88,0x84,0x88,0xf0,         // 127 	[0x7f]
0x00,0x00,0x00,0x00,0x00,         // 128 	[0x80]
0xfe,0xff,0x33,0xff,0xfe,         // 129 	[0x81]
0xff,0xff,0xdb,0xff,0x76,         // 130 	[0x82]
0x7e,0xff,0xc3,0xc3,0xc3,         // 131 	[0x83]
0xff,0xff,0xc3,0xff,0x7e,         // 132 	[0x84]
0xff,0xff,0xdb,0xdb,0xc3,         // 133 	[0x85]
0xff,0xff,0x1b,0x1b,0x03,         // 134 	[0x86]
0x7e,0xff,0xd3,0xf7,0x76,         // 135 	[0x87]
0xff,0xff,0x18,0xff,0xff,         // 136 	[0x88]
0x00,0xff,0xff,0xff,0x00,         // 137 	[0x89]
0x60,0xe0,0xc0,0xff,0x7f,         // 138 	[0x8a]
0xff,0xff,0x3c,0xf7,0xe3,         // 139 	[0x8b]
0xff,0xff,0xc0,0xc0,0xc0,         // 140 	[0x8c]
0xff,0x07,0x1c,0x07,0xff,         // 141 	[0x8d]
0xff,0x3f,0x7e,0xfc,0xff,         // 142 	[0x8e]
0x7e,0xff,0xc3,0xff,0x7e,         // 143 	[0x8f]
0xff,0xff,0x33,0x3f,0x1e,         // 144 	[0x90]
0x7e,0xff,0xe3,0x7f,0xbe,         // 145 	[0x91]
0xff,0xff,0x3b,0xef,0xc6,         // 146 	[0x92]
0xce,0xdf,0xdb,0xfb,0x73,         // 147 	[0x93]
0x03,0xff,0xff,0xff,0x03,         // 148 	[0x94]
0x7f,0xff,0xc0,0xff,0x7f,         // 149 	[0x95]
0x3f,0x7f,0xe0,0x7f,0x3f,         // 150 	[0x96]
0xff,0xe0,0x38,0xe0,0xff,         // 151 	[0x97]
0xc7,0xff,0x38,0xff,0xc7,         // 152 	[0x98]
0x0f,0x1f,0xf0,0x1f,0x0f,         // 153 	[0x99]
0xe3,0xf3,0xdb,0xcf,0xc7,         // 154 	[0x9a]
0x00,0x00,0x00,0x00,0x00,         // 155 	[0x9b]
0x00,0x00,0x00,0x00,0x00,         // 156 	[0x9c]
0x00,0x00,0x00,0x00,0x00,         // 157 	[0x9d]
0x00,0x00,0x00,0x00,0x00,         // 158 	[0x9e]
0x00,0x00,0x00,0x00,0x00,         // 159 	[0x9f]
0x00,0x00,0x00,0x00,0x00,         // 160 	[0xa0]
0x00,0x00,0x00,0x00,0x00,         // 161 	[0xa1]
0x00,0x00,0x00,0x00,0x00,         // 162 	[0xa2]
0x00,0x00,0x00,0x00,0x00,         // 163 	[0xa3]
0x00,0x00,0x00,0x00,0x00,         // 164 	[0xa4]
0x00,0x00,0x00,0x00,0x00,         // 165 	[0xa5]
0x00,0x00,0x00,0x00,0x00,         // 166 	[0xa6]
0x00,0x00,0x00,0x00,0x00,         // 167 	[0xa7]
0x00,0x00,0x00,0x00,0x00,         // 168 	[0xa8]
0x00,0x00,0x00,0x00,0x00,         // 169 	[0xa9]
0x00,0x00,0x00,0x00,0x00,         // 170 	[0xaa]
0x00,0x00,0x00,0x00,0x00,         // 171 	[0xab]
0x00,0x00,0x00,0x00,0x00,         // 172 	[0xac]
0x00,0x00,0x00,0x00,0x00,         // 173 	[0xad]
0x00,0x00,0x00,0x00,0x00,         // 174 	[0xae]
0x00,0x00,0x00,0x00,0x00,         // 175 	[0xaf]
0x00,0x00,0x00,0x00,0x00,         // 176 	[0xb0]
0x00,0x00,0x00,0x00,0x00,         // 177 	[0xb1]
0x00,0x00,0x00,0x00,0x00,         // 178 	[0xb2]
0x00,0x00,0x00,0x00,0x00,         // 179 	[0xb3]
0x00,0x00,0x00,0x00,0x00,         // 180 	[0xb4]
0x00,0x00,0x00,0x00,0x00,         // 181 	[0xb5]
0x00,0x00,0x00,0x00,0x00,         // 182 	[0xb6]
0x00,0x00,0x00,0x00,0x00,         // 183 	[0xb7]
0x00,0x00,0x00,0x00,0x00,         // 184 	[0xb8]
0x00,0x00,0x00,0x00,0x00,         // 185 	[0xb9]
0x00,0x00,0x00,0x00,0x00,         // 186 	[0xba]
0x00,0x00,0x00,0x00,0x00,         // 187 	[0xbb]
0x00,0x00,0x00,0x00,0x00,         // 188 	[0xbc]
0x00,0x00,0x00,0x00,0x00,         // 189 	[0xbd]
0x00,0x00,0x00,0x00,0x00,         // 190 	[0xbe]
0x00,0x00,0x00,0x00,0x00,         // 191 	[0xbf]
0x00,0x00,0x00,0x00,0x00,         // 192 	[0xc0]
0x00,0x00,0x00,0x00,0x00,         // 193 	[0xc1]
0x00,0x00,0x00,0x00,0x00,         // 194 	[0xc2]
0x00,0x00,0x00,0x00,0x00,         // 195 	[0xc3]
0x00,0x00,0x00,0x00,0x00,         // 196 	[0xc4]
0x00,0x00,0x00,0x00,0x00,         // 197 	[0xc5]
0x00,0x00,0x00,0x00,0x00,         // 198 	[0xc6]
0x00,0x00,0x00,0x00,0x00,         // 199 	[0xc7]
0x00,0x00,0x00,0x00,0x00,         // 200 	[0xc8]
0x00,0x00,0x00,0x00,0x00,         // 201 	[0xc9]
0x00,0x00,0x00,0x00,0x00,         // 202 	[0xca]
0x00,0x00,0x00,0x00,0x00,         // 203 	[0xcb]
0x00,0x00,0x00,0x00,0x00,         // 204 	[0xcc]
0x00,0x00,0x00,0x00,0x00,         // 205 	[0xcd]
0x00,0x00,0x00,0x00,0x00,         // 206 	[0xce]
0x00,0x00,0x00,0x00,0x00,         // 207 	[0xcf]
0x00,0x00,0x00,0x00,0x00,         // 208 	[0xd0]
0x00,0x00,0x00,0x00,0x00,         // 209 	[0xd1]
0x00,0x00,0x00,0x00,0x00,         // 210 	[0xd2]
0x00,0x00,0x00,0x00,0x00,         // 211 	[0xd3]
0x00,0x00,0x00,0x00,0x00,         // 212 	[0xd4]
0x00,0x00,0x00,0x00,0x00,         // 213 	[0xd5]
0x00,0x00,0x00,0x00,0x00,         // 214 	[0xd6]
0x00,0x00,0x00,0x00,0x00,         // 215 	[0xd7]
0x00,0x00,0x00,0x00,0x00,         // 216 	[0xd8]
0x00,0x00,0x00,0x00,0x00,         // 217 	[0xd9]
0x00,0x00,0x00,0x00,0x00,         // 218 	[0xda]
0x00,0x00,0x00,0x00,0x00,         // 219 	[0xdb]
0x00,0x00,0x00,0x00,0x00,         // 220 	[0xdc]
0x00,0x00,0x00,0x00,0x00,         // 221 	[0xdd]
0x00,0x00,0x00,0x00,0x00,         // 222 	[0xde]
0x00,0x00,0x00,0x00,0x00,         // 223 	[0xdf]
0x00,0x00,0x00,0x00,0x00,         // 224 	[0xe0]
0x00,0x00,0x00,0x00,0x00,         // 225 	[0xe1]
0x00,0x00,0x00,0x00,0x00,         // 226 	[0xe2]
0x00,0x00,0x00,0x00,0x00,         // 227 	[0xe3]
0x00,0x00,0x00,0x00,0x00,         // 228 	[0xe4]
0x00,0x00,0x00,0x00,0x00,         // 229 	[0xe5]
0x00,0x00,0x00,0x00,0x00,         // 230 	[0xe6]
0x00,0x00,0x00,0x00,0x00,         // 231 	[0xe7]
0x00,0x00,0x00,0x00,0x00,         // 232 	[0xe8]
0x04,0x54,0x50,0x14,0x44,         // 233 	[0xe9]
0x00,0x00,0x00,0x00,0x00,         // 234 	[0xea]
0x00,0x00,0x00,0x00,0x00,         // 235 	[0xeb]
0x10,0x22,0x10,0x22,0x10,         // 236 	[0xec]
0x10,0x22,0x20,0x22,0x10,         // 237 	[0xed]
0x20,0x22,0x20,0x22,0x20,         // 238 	[0xee]
0x40,0x22,0x20,0x22,0x40,         // 239 	[0xef]
0x60,0xff,0xf9,0xdf,0x60,         // 240 	[0xf0]
0x0c,0x1f,0xfc,0x1f,0x0c,         // 241 	[0xf1]
0x00,0x00,0x00,0x00,0x00,         // 242 	[0xf2]
0x00,0x00,0x00,0x00,0x00,         // 243 	[0xf3]
0x00,0x00,0x00,0x00,0x00,         // 244 	[0xf4]
0x00,0x00,0x00,0x00,0x00,         // 245 	[0xf5]
0x00,0x00,0x00,0x00,0x00,         // 246 	[0xf6]
0x00,0x00,0x00,0x00,0x00,         // 247 	[0xf7]
0x00,0x00,0x00,0x00,0x00,         // 248 	[0xf8]
0xfe,0x7b,0xff,0x7b,0xfe,         // 249 	[0xf9]
0xfe,0x7a,0xfe,0x7a,0xfe,         // 250 	[0xfa]
0xfc,0x78,0xfc,0x78,0xfc,         // 251 	[0xfb]
0xf8,0x78,0xf8,0x78,0xf8,         // 252 	[0xfc]
0xf0,0x70,0xf0,0x70,0xf0,         // 253 	[0xfd]
0xe0,0x60,0xe0,0x60,0xe0,         // 254 	[0xfe]
0xc0,0x40,0xc0,0x40,0xc0          // 255 	[0xff]
;

const unsigned char screen_1541_III[504] =			/* Image Array Table Name, Width, Height In Lines, 506 bytes*/
0x00,0xe0,0x20,0xa0,0xa0,0xa0,0x20,0xa0,0xa0,0xa0,0x20,0xa0,0xa0,0x40,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0xfc,0x04,0x04,0x84,0x04,0xf4,0x94,0x64,0x04,0xf4,0xa4,0xe4,0x04,0x74,0x44,0xf4,0x44,0x04,0x04,0xfc,0xfc,0x00,0x00,0x00,0x10,0xf0,0x10,0x00,0xf0,0x50,0x10,0x00,0xe0,0x10,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x80,0x80,0x80,0x80,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0x00,0x43,0x30,0x0b,0x00,0x5b,0x68,0x03,0x78,0x4b,0x32,0x00,0x00,0xff,0x00,0x20,0x20,0xa8,0x70,0x20,0x00,0xfe,0xff,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xce,0x86,0x86,0x86,0xce,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xff,0xfe,0x00,0x70,0x88,0x71,0x01,0xf9,0x00,0xf9,0x01,0xf9,0x00,0x70,0x89,0x71,0x00,0xf8,0x00,0x20,0x20,0xa8,0x70,0x20,0x00,0xf8,0xfe,0xff,0x8f,0x07,0x03,0x03,0x07,0x07,0x07,0xd8,0xd8,0xd8,0xd8,0x88,0x00,
0x00,0x1f,0x10,0x1e,0x10,0x1e,0x10,0x1e,0x10,0x1e,0x10,0x1e,0x10,0x1e,0x10,0x1f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x43,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x07,0x0f,0x0f,0x0e,0x0e,0x0f,0x0f,0x07,0x00,0x00,0x00,0x00,0x00,0x00,
0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0x00,0xe0,0xb0,0x10,0xf0,0x00,0xf0,0x10,0xd0,0x50,0x50,0x70,0x00,0x80,0xe0,0x10,0xf0,0x00,0x00,0x00,0xe0,0xb0,0x10,0xf0,0x00,0x00,0x00,0x00,0x00,0x70,0xd0,0x10,0xd0,0x50,0xd0,0x10,0xd0,0x50,0xd0,0x10,0xd0,0x70,0x00,0x00,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,0xd0,
0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0x00,0x00,0xff,0x80,0xff,0x00,0x77,0x94,0xa5,0xbd,0x83,0x7e,0x00,0x3f,0x20,0x2f,0xe9,0x84,0xec,0x18,0x00,0xff,0x80,0xff,0x00,0x0e,0x0a,0x0a,0x0e,0xe0,0xbf,0x80,0xbf,0xa0,0xbf,0x80,0xbf,0xa0,0xbf,0x80,0xbf,0xe0,0x00,0x00,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,0xee,
0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0x00,0x00,0x7c,0x50,0x20,0x0c,0x50,0x3c,0x00,0x00,0x00,0x20,0x40,0x3c,0x00,0x40,0x00,0x7c,0x44,0x38,0x00,0x7c,0x54,0x00,0x7c,0x14,0x68,0x00,0x38,0x44,0x38,0x00,0x38,0x44,0x54,0x34,0x00,0x7c,0x54,0x00,0x7c,0x54,0x00,0x00,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd,0xfd;

const unsigned char screen_1541_III_DTV[504] =		/* Image Array Table Name, Width, Height In Lines, 506 bytes*/
0x00,0xe0,0x20,0xa0,0xa0,0xa0,0x20,0xa0,0xa0,0xa0,0x20,0xa0,0xa0,0x40,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfc,0xfc,0x04,0x04,0x84,0x04,0xf4,0x94,0x64,0x04,0xf4,0xa4,0xe4,0x04,0x74,0x44,0xf4,0x44,0x04,0x04,0xfc,0xfc,0x00,0x00,0x00,0x10,0xf0,0x10,0x00,0xf0,0x50,0x10,0x00,0xe0,0x10,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x40,0x20,0x20,0xa0,0x40,0x80,0x00,0x00,0x00,0x00,0x00,
0x00,0xff,0x00,0x43,0x30,0x0b,0x00,0x5b,0x68,0x03,0x78,0x4b,0x32,0x00,0x00,0xff,0x00,0x20,0x20,0xa8,0x70,0x20,0x00,0xfe,0xff,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xce,0x86,0x86,0x86,0xce,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xff,0xfe,0x00,0x70,0x88,0x71,0x01,0xf9,0x00,0xf9,0x01,0xf9,0x00,0x70,0x89,0x71,0x00,0xf8,0x00,0x20,0x20,0xa8,0x70,0x20,0x00,0x00,0x00,0x00,0x00,0x03,0x04,0xf8,0x08,0xf8,0x04,0x03,0x00,0x00,0x00,0x00,0x00,
0x00,0x1f,0x10,0x1e,0x10,0x1e,0x10,0x1e,0x10,0x1e,0x10,0x1e,0x10,0x1e,0x10,0x1f,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x43,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x7f,0x00,0x00,0x00,0x40,0x40,0x40,0x80,0x80,0x40,0x40,0x20,0x20,0x10,0x10,0x20,0x20,0x40,0x40,0x80,0x80,0x80,0x40,0x40,0x5c,0x42,0x23,0x23,0x13,0x52,0x53,0x52,0x53,0x42,0x43,0x43,0x43,0x62,0x3c,0x00,
0xd0,0xd0,0xd0,0x00,0x00,0x00,0xe0,0xb0,0x10,0xf0,0x00,0xf0,0x10,0xd0,0x50,0x50,0x70,0x00,0x80,0xe0,0x10,0xf0,0x00,0x00,0x00,0xe0,0xb0,0x10,0xf0,0x00,0x00,0x00,0x00,0x00,0x70,0xd0,0x10,0xd0,0x50,0xd0,0x10,0xd0,0x50,0xd0,0x10,0xd0,0x70,0x00,0x00,0x00,0x00,0x00,0xf0,0x10,0xd0,0x50,0x90,0x20,0xc0,0x00,0x70,0x50,0xd0,0x10,0xd0,0x50,0x70,0x00,0x70,0x90,0x70,0x80,0x00,0x80,0x70,0x90,0x70,0x00,0x00,0x00,0xd0,0xd0,0xd0,0xd0,
0xee,0xee,0xee,0x00,0x00,0x00,0x00,0xff,0x80,0xff,0x00,0x77,0x94,0xa5,0xbd,0x83,0x7e,0x00,0x3f,0x20,0x2f,0xe9,0x84,0xec,0x18,0x00,0xff,0x80,0xff,0x00,0x0e,0x0a,0x0a,0x0e,0xe0,0xbf,0x80,0xbf,0xa0,0xbf,0x80,0xbf,0xa0,0xbf,0x80,0xbf,0xe0,0x00,0x00,0x00,0x00,0x00,0xff,0x80,0xbf,0xa0,0x9f,0x40,0x3f,0x00,0x00,0x00,0xff,0x80,0xff,0x00,0x00,0x00,0x00,0x0f,0x70,0x8f,0xb0,0x8f,0x70,0x0f,0x00,0x00,0x00,0x00,0xee,0xee,0xee,0xee,
0xfd,0xfd,0xfd,0x00,0x4c,0x74,0x00,0x38,0x44,0x38,0x00,0x78,0x14,0x00,0x3c,0x50,0x00,0x3c,0x40,0x20,0x40,0x3c,0x00,0x78,0x14,0x78,0x00,0x7c,0x14,0x68,0x00,0x7c,0x54,0x00,0x00,0x00,0x00,0x00,0x7c,0x54,0x28,0x00,0x0c,0x70,0x0c,0x00,0x00,0x00,0x20,0x40,0x3c,0x00,0x40,0x00,0x7c,0x44,0x38,0x00,0x7c,0x54,0x00,0x7c,0x14,0x68,0x00,0x38,0x44,0x38,0x00,0x38,0x44,0x54,0x34,0x00,0x7c,0x54,0x00,0x7c,0x54,0x00,0xfd,0xfd,0xfd,0xfd;

/*--------------------------------------------------------*/
/*                         macro's                        */
/*--------------------------------------------------------*/

/*--------------------------------------------------------*/
/*                         globals                        */
/*--------------------------------------------------------*/
extern unsigned char setting_contrast;

bit invert_characters;
bit bold_characters;

/*--------------------------------------------------------*/
/*                     local functions                    */
/*--------------------------------------------------------*/
void Send_byte(unsigned char data);
void Lcd_char(unsigned char data);
void Lcd_data(unsigned char data);

/*--------------------------------------------------------*/
void InitLcd(void)
{
	NOKIA_SCL = 0;
	NOKIA_SDA = 0;
	NOKIA_SX = 1;		    	
	NOKIA_DX = 0;		      	

	NOKIA_RST = 0;    				/*nokia display reset*/
	DelayMs(10);				
	NOKIA_RST = 1;    			

	NOKIA_SX = 0;		    		/*enable display*/
	Lcd_command(0x21);				/*horizontal_addressing, go into extended instruction set mode*/
	Lcd_command(setting_contrast);	/*C5, set LCD Vop (80=lowest contrast voltage, FF=highest)*/
	Lcd_command(0x06);				/*set temp coef.*/
	Lcd_command(0x13);				/*set bias*/
	Lcd_command(0x20);				/*basic instruction set, horizontal addr. mode*/
	Lcd_command(0x0c);				/*display in normal mode*/
}

void SetContrast(void)
{
	InitLcd();						/*dit moet toch mooier kunnen... <zucht>*/
}

void Lcd_clr(void)
{
	unsigned int bytecounter;

	Lcd_XY_address(0,0);

	bytecounter = 612;				/* originally 504, but the extra values just wrap around and cause no harm!    */
									/* some non-original displays seem to have larger display memory than standard */
	while(bytecounter--)
	{
		Lcd_data(0x00);
	}
}

void Lcd_show_1541_III(void)
{
	unsigned int bytecounter, rowcounter, row;

	Lcd_XY_address(0,0);

	row = 0;					/* y-position on screen (0-5)  */
	rowcounter = 0;				/* x-position on screen (0-83) */
	bytecounter = 0;
	Lcd_data(screen_1541_III[bytecounter]);
	while(bytecounter != 504)	
	{
		if (rowcounter == 83)   /* if we're over the last position, line feed to beginning of next row */
		{
			rowcounter=0;
			row=row+1;
			Lcd_XY_address(0,row);
		}
		else rowcounter++;
		
		bytecounter++;
		Lcd_data(screen_1541_III[bytecounter]);
	}
} 

void Lcd_show_1541_III_DTV(void)
{
	unsigned int bytecounter, rowcounter, row;

	Lcd_XY_address(0,0);

	row = 0;					/* y-position on screen (0-5)  */
	rowcounter = 0;				/* x-position on screen (0-83) */
	bytecounter = 0;
	Lcd_data(screen_1541_III_DTV[bytecounter]);
	while(bytecounter != 504)	
	{
		if (rowcounter == 83)   /* if we're over the last position, line feed to beginning of next row */
		{
			rowcounter=0;
			row=row+1;
			Lcd_XY_address(0,row);
		}
		else rowcounter++;
		
		bytecounter++;
		Lcd_data(screen_1541_III_DTV[bytecounter]);
	}
} 

void Lcd_XY_address(unsigned char X, unsigned char Y)
{
	Lcd_command(0x80 + (6*X));		/*set X-address*/
	Lcd_command(0x40 + Y);			/*set Y-address*/
}

void Send_byte(unsigned char data)
{
	unsigned char bitcount;
	bitcount = 8;

	while (bitcount != 0)			/*data is transmitted with LSB first*/
	{
		NOKIA_SCL = 0;
		DelayUs(1);	
		bitcount--;
		if (data & 0x80)
		{
			NOKIA_SDA = 1;
		}
		else
		{		
			NOKIA_SDA = 0;
		}
		NOKIA_SCL = 1;
		DelayUs(1);	
		NOKIA_SDA = 0;			/*make the data-line 0, if we don't we'd get contrast problem when we use to much black i.e. inverted line of characters would blank out the entire display*/
		data<<=1;				/*shift the byte one bit to the left*/
	}
}

void Lcd_command (unsigned char command)
{
	NOKIA_DX = 0;		      	/*nokia display control-mode*/
	Send_byte(command);
}

void Lcd_data (unsigned char data)
{
	NOKIA_DX = 1;		      	/*nokia display data-mode*/
	Send_byte(data);
}

void Lcd_CharInvert(void)
{
	invert_characters = TRUE;
}

void Lcd_CharNormal(void)
{
	bold_characters = FALSE;
	invert_characters = FALSE;
}

void Lcd_CharBold(void)
{
	bold_characters = TRUE;
}

void Lcd_char(unsigned char data)
{
	if (bold_characters == TRUE)
	{
		if ((data >= 'A') && (data <= 'Z'))
		{
			data = data + 0x40;
		}
	}

	if (invert_characters == FALSE)
	{
		Lcd_data(CharFont[data][0]);
		Lcd_data(CharFont[data][1]);
		Lcd_data(CharFont[data][2]);
		Lcd_data(CharFont[data][3]);
		Lcd_data(CharFont[data][4]);
		Lcd_data(0x00);
	}
	else
	{
//	als ik de FE vervang door FF dan word het scherm wit... totdat ik reset, update ik het scherm te snel?
		Lcd_data(0xFf - CharFont[data][0]);
		Lcd_data(0xFf - CharFont[data][1]);
		Lcd_data(0xFf - CharFont[data][2]);
		Lcd_data(0xFf - CharFont[data][3]);
		Lcd_data(0xFf - CharFont[data][4]);
		Lcd_data(0xFf);
	}
}

/*a conventional progressbar, that is empty at 0% and full at 100%*/
void Lcd_ProgressBar(unsigned int value, unsigned int max_value)
{
	#define	bar_total_size	82

	unsigned int lp;
	unsigned int size;

	size = (value * bar_total_size) / max_value;

	Lcd_data(0x7E);
	lp = bar_total_size;
	while(lp--)
	{
		if ((lp<size) || (max_value == 0))
			Lcd_data(0x42);
		else
			Lcd_data(0x7E);
	}
	Lcd_data(0x7E);
}

/*this bar has a block that moves, the bar will never be filled completely, compare it to a slider in Windows*/
void Lcd_SelectionBar(unsigned int value, unsigned int max_value)
{
	#define	bar_total_size	82

	unsigned int lp;
	unsigned int size_start;
	unsigned int size_end;

	size_start = (value * bar_total_size) / max_value;
	size_end = size_start + (bar_total_size/max_value);

	Lcd_data(0x7E);
	lp = bar_total_size;
	while(lp--)
	{
		if ((lp<size_start) || (lp>size_end))
		{
			Lcd_data(0x42);
		}
		else
		{
			Lcd_data(0x7E);
		}
	}
	Lcd_data(0x7E);
}
