/*----------------------------------------------------------------------------------*/
/* Sound routines. Based on the BTC project by...            */
/*----------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------*/

/*  History:                                                                        
    --------    
	2009-07-15	okay, 3 years have passed, with new knowledge and lack of space for relevant software routines. The sound samples must leave.
				The samples are replaced by synthetic sounds. Wow! sounds nice. Nope, only in theory, in real live it sounds cheap. But it's better then nothing.
				And the alternative it throwing away to much program space.
				Maybe I'll finetune it later...
	2006-07-31	finally accepted the limmitations of the (great) btc principle, now accepted that more bits and a PWM out can do the trick with only a speaker and a 100uF capacitor
	2006-07-..	tried btc, but this did not give the expected quality required for realistic 1541-sound
	2006-07-25	first attempt
*/

/*  TO DO:                                                                          
    ------      
*/

/*----------------------------------------------------------------------------------*/


/*--------------------------------------------------------*/
/*                        includes                        */
/*--------------------------------------------------------*/
#include <pic18.h>
#include <hardware.h>
#include <sound.h>
#include <delay.h>

/*--------------------------------------------------------*/
/*                         globals                        */
/*--------------------------------------------------------*/


/*--------------------------------------------------------*/
/*                       Sound table                      */
/*--------------------------------------------------------*/

/*--------------------------------------------------------*/
void HeadStep(unsigned char steps, unsigned char freq);

/*****************************************************************************************************************/
/*****************************************************************************************************************/

//	lp = rand();		/*generate a semi random number, the sequence of number is always*/

void Sound_Blip(void)
{
	unsigned char	sawtooth;					/*counter to generate the waveform*/
	unsigned char	duration;
	
	duration = 50;
	CCPR2L = sawtooth++;

	while(duration > 0)
	{
		CCPR2L = sawtooth++;					/*send it to the PWM output*/
		if (sawtooth > 200)						/*this value determines the max volume as well as the frequency*/
		{
			sawtooth = 0;
			duration --;
		}
		asm("nop");								/*a small delay for the porper tuning*/
		asm("nop");								/*a small delay for the porper tuning*/
		asm("nop");								/*a small delay for the porper tuning*/
		asm("nop");								/*a small delay for the porper tuning*/
		asm("nop");								/*a small delay for the porper tuning*/
		asm("nop");								/*a small delay for the porper tuning*/
	}
}

void Sound_HeadBangRattle(void)
{
	unsigned char	lp;
	unsigned char 	freq;
	
	freq = 75;
	for(lp=0; lp<20; lp++)
	{
		freq++;
		freq++;
		HeadStep(1, freq);
	}
	DelayMs(100);

	for(lp=0; lp<20; lp++)
	{
		freq--;
		freq--;
		HeadStep(1, freq);
	}
}


/*the following routine describes the sound of a track change*/
void Sound_TrackChange(int numberoftracks)
{
	if (numberoftracks >= 0)
		HeadStep(numberoftracks, 75);	
	else
		HeadStep((0-numberoftracks), 65);						
}

/*create the head-stepping "click"-sound of the real drive*/
/*the freq value is for the frequency of the "click" sound. It's just a detail, but it sounds better*/
/*75 is a nice value*/
void HeadStep(unsigned char steps, unsigned char freq)
{
	unsigned char lp;

	while(steps > 0)
	{
		steps--;
		for(lp=0; lp<8; lp++)	/*this is the sound of 1 step*/
		{						/**/
			CCPR2L = 0;			/**/
			DelayBigUs(freq);	/**/
			CCPR2L = 255;		/**/
			DelayBigUs(freq);	/**/
			CCPR2L = 0;			/**/
		}						/**/
		DelayBigUs(17500);		/*this is the delay between each step-sound, this makes it sound like a rattle when more then one step is made, just like the real drive*/
	}
}
